<?php
/**
 * Ce fichier contient l'ensemble des constantes et fonctions implémentant le type de service GEOMETRIE.
 * Les services de GEOMETRIE sont des mécanisme internes à Nomenclatures pour
 * charger les différents ensemble de contours géographiques fournis par des services divers et sous des
 * formats variés (fichiers geoJSON, topoJSON, API REST...).
 *
 * @package SPIP\ISOCODE\SERVICES\GEOMETRIE
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

require_once _DIR_PLUGIN_ISOCODE . 'vendor/autoload.php';
use Vicchi\GeoJson\Rewind;

$GLOBALS['isocode']['geometrie'] = array(
	'countries' => array(
		'table' => 'geoboundaries',
		'type'  => 'country',
		'pays'  => '',
		'basic_fields' => array(
			'iso_a2'        => 'code',
			'geo_shape'     => 'geometry',
			'geo_point_2d'  => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=natural-earth-countries-1_50m&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'iso_a2'        => 'fields/iso_a2',
			'geo_shape'     => 'fields/geo_shape',
			'geo_point_2d'  => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'natural-earth-countries-1_50m',
			'provider_name' => 'Natural Earth',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/natural-earth-countries-1_50m/information/',
			'license'       => 'Domaine Public',
			'last_update'   => '08/03/2018'
		),
	),
	'frregion' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'FR',
		'basic_fields' => array(
			'reg_code' => 'code',
			'geo_shape'  => 'geometry',
			'geo_point_2d'  => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_insee_reg',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-france-region&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'reg_code'      => 'fields/reg_code',
			'geo_shape'     => 'fields/geo_shape',
			'geo_point_2d'  => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-region',
			'provider_name' => 'IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-france-region/information/?disjunctive.reg_name',
			'license'       => 'Licence Ouverte 2.0 (Etalab)',
			'last_update'   => '01/12/2020'
		),
	),
	'frdepartment' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'FR',
		'basic_fields' => array(
			'dep_code' => 'code',
			'geo_shape'  => 'geometry',
			'geo_point_2d'  => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_insee',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-france-departement&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'dep_code'      => 'fields/dep_code',
			'geo_shape'     => 'fields/geo_shape',
			'geo_point_2d'  => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-departement',
			'provider_name' => 'IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-france-departement/information/?disjunctive.reg_name',
			'license'       => 'Licence Ouverte 2.0 (Etalab)',
			'last_update'   => '22/06/2020'
		),
	),
	'deland' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'DE',
		'basic_fields' => array(
			'lan_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_ars',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-germany-land&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'lan_code'     => 'fields/lan_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-departement',
			'provider_name' => 'GeoBasis-DE / BKG 2020',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-germany-land/information/?disjunctive.reg_name',
			'license'       => 'Data licence Germany – attribution – version 2.0',
			'last_update'   => '11/09/2020'
		),
	),
	'chcanton' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'CH',
		'basic_fields' => array(
			'kan_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_ktnr',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-switzerland-kanton&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'kan_code'     => 'fields/kan_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-switzerland-kanton',
			'provider_name' => 'Office fédéral de topographie swisstopo',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-switzerland-kanton/information/?disjunctive.kan_code&disjunctive.kan_name&sort=year',
			'license'       => 'Opendata.swiss BY ASK',
			'last_update'   => '05/01/2021'
		),
	),
	'beregion' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'BE',
		'basic_fields' => array(
			'ins'          => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_ins_reg',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=regions-belges&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'ins'          => 'fields/ins',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'regions-belges',
			'provider_name' => 'Geomatics Unit - ULg, NGI/IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-switzerland-kanton/information/?disjunctive.kan_code&disjunctive.kan_name&sort=year',
			'license'       => 'Licence spécifique',
			'last_update'   => '22/02/2018'
		),
	),
	'beprovince' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'BE',
		'basic_fields' => array(
			'proviso'      => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=provinces-belges&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'proviso'      => 'fields/proviso',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'provinces-belges',
			'provider_name' => 'Geomatics Unit - ULg, NGI/IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-switzerland-kanton/information/?disjunctive.kan_code&disjunctive.kan_name&sort=year',
			'license'       => 'Licence spécifique',
			'last_update'   => '22/02/2018'
		),
	),
	'usstate' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'US',
		'basic_fields' => array(
			'stusab'       => 'code',
			'st_asgeojson' => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=us-state-boundaries&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'stusab'       => 'fields/stusab',
			'st_asgeojson' => 'fields/st_asgeojson',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'us-state-boundaries',
			'provider_name' => 'U.S. Census Bureau',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/us-state-boundaries/information/?flg=fr',
			'license'       => 'Domaine public',
			'last_update'   => '24/10/2019'
		),
	),
	'nlprovince' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'NL',
		'basic_fields' => array(
			'province_id'  => 'code',
			'geometry'     => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=auadministrativeunit-provinces&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'province_id'  => 'fields/province_id',
			'geometry'     => 'fields/geometry',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'auadministrativeunit-provinces',
			'provider_name' => 'PDOK',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/auadministrativeunit-provinces/information/?flg=fr',
			'license'       => 'Domaine public',
			'last_update'   => '24/10/2019'
		),
	),
	'ptdistrict' => array(
		'table' => 'geoboundaries',
		'type'  => 'subdivision',
		'pays'  => 'PT',
		'basic_fields' => array(
			'dis_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		),
		'basic_ext_fields' => array(
			'lat',
			'lon',
		),
		'static_fields' => array(
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		),
		'unused_fields' => array(
			'lat_lon' => '',
		),
		'populating'   => 'api_rest',
		'url'          => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-portugal-distrito&q=&rows=-1',
		'extension'    => '.json',
		'node'         => 'records',
		'basic_nodes'  => array(
			'dis_code'     => 'fields/dis_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		),
		'credits' => array(
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-portugal-distrito',
			'provider_name' => 'Direção-Geral do Território',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-portugal-distrito/information/?disjunctive.dis_code&disjunctive.dis_name&sort=year',
			'license'       => 'Domaine public',
			'last_update'   => '07/01/2021'
		),
	),
);

// ----------------------------------------------------------------------------
// ---------- API du type de service GEOMETRIE - Actions principales ----------
// ----------------------------------------------------------------------------

function countries_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);


	// Et d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

function frregion_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

function frdepartment_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

function deland_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

function chcanton_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

function beregion_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code INS sur 5 digits car il est fourni par l'API sans le 0 en premier caractère
	$element['ins'] = str_pad($element['ins'], 5, '0', STR_PAD_LEFT);

	return $element;
}

function beprovince_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code ISO en lui rajoutant le code ISO-3166 alpha2 de la Belgique
	$element['proviso'] = $element['country'] . '-' . $element['proviso'];

	return $element;
}

function usstate_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['st_asgeojson'] = json_encode(Rewind::rewind($element['st_asgeojson'], true));

	// Formater le code ISO en lui rajoutant le code ISO-3166 alpha2 des US
	$element['stusab'] = $element['country'] . '-' . $element['stusab'];

	return $element;
}

function nlprovince_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geometry'] = json_encode(Rewind::rewind($element['geometry'], true));

	// Le champ provice_id est identique à la partie spécifique du code ISO 3166-2 sauf pour le brabant nord et
	// la Hollande du nord.
	// -- on passe l'id en majuscules
	// -- on ajoute le code du pays et on change le code de la province pour les 2 provinces spécifiées.
	$province_id = strtoupper($element['province_id']);
	if ($province_id === 'HO') {
		$province_id = 'NH';
	} elseif ($province_id === 'BR') {
		$province_id = 'NB';
	}
	$element['province_id'] = $element['country'] . '-' . $province_id;

	return $element;
}

function ptdistrict_completer_element($element, $config) {

	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = floatval($element['geo_point_2d'][0]);
	$element['lon'] = floatval($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code ISO en lui rajoutant le code ISO-3166 alpha2 de la Belgique
	$element['dis_code'] = $element['country'] . '-' . $element['dis_code'];

	return $element;
}
