<?php
/**
 * Ce fichier contient l'ensemble des constantes et fonctions implémentant le type de service INFRASUBDIVISION.
 * Les services de INFRASUBDIVISION sont des mécanisme internes à Nomenclatures pour
 * charger les codes nationaux des infra subdivisions fournis par des services divers.
 *
 * @package SPIP\ISOCODE\SERVICES\INFRASUBDIVISION
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}


$GLOBALS['isocode']['infrasubdivision'] = array(
	'inseearrond' => array(
		'table' => 'infrasubdivisions',
		'type'  => 'infra_subdivision',
		'pays'  => 'FR',
		'basic_fields' => array(
			'arr'    => 'code',
			'dep'    => 'parent',
			'nccenr' => 'label_fr',
		),
		'static_fields' => array(
			'service'   => '$service',
			'type'      => 'arrondissement',
			'country'   => '/pays',
		),
		'depth_field'   => array(
			'name'         => 'depth',
			'base_value'   => 0,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		),
		'unused_fields' => array(
			'label_fr'  => '',
		),
		'label_field'  => true,
		'populating'   => 'file_csv',
		'multiple'     => true,
		'delimiter'    => ",",
		'extension'    => '.csv',
	),
	'inseecommune' => array(
		'table' => 'infrasubdivisions',
		'type'  => 'infra_subdivision',
		'pays'  => 'FR',
		'basic_fields' => array(
			'typecom'   => 'type',
			'com'       => 'code',
			'arr'       => 'parent',
			'nccenr'    => 'label_fr',
			'can'       => 'can',
			'comparent' => 'comparent',
		),
		'static_fields' => array(
			'service'   => '$service',
			'country'   => '/pays',
		),
		'depth_field'   => array(
			'name'         => 'depth',
			'base_value'   => 1,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		),
		'unused_fields' => array(
			'can'       => '',
			'comparent' => '',
			'label_fr'  => '',
		),
		'label_field'  => true,
		'populating'   => 'file_csv',
		'multiple'     => true,
		'delimiter'    => ",",
		'extension'    => '.csv',
	),
	'uscbcounty' => array(
		'table' => 'infrasubdivisions',
		'type'  => 'infra_subdivision',
		'pays'  => 'US',
		'basic_fields' => array(
			'code'      => 'code',
			'state'     => 'parent',
			'name'      => 'label_en',
			'entity'    => 'type',
		),
		'static_fields' => array(
			'service'   => '$service',
			'country'   => '/pays',
		),
		'depth_field'   => array(
			'name'         => 'depth',
			'base_value'   => 0,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		),
		'unused_fields' => array(
			'label_en'  => '',
		),
		'label_field'  => true,
		'populating'   => 'file_csv',
		'multiple'     => true,
		'delimiter'    => ";",
		'extension'    => '.csv',
	),
);

// -----------------------------------------------------------------------------------
// ---------- API du type de service INFRASUBDIVISION - Actions principales ----------
// -----------------------------------------------------------------------------------

function inseearrond_completer_element($element, $config) {

	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - modifier l'identifiant de l'arrondissement
	$element['arr'] = 'FR-' . $element['arr'];
	// - Modifier l'identifiant du parent qui est une subdivision
	$element['dep'] = 'FR-' . $element['dep'];

	return $element;
}

function inseecommune_completer_element($element, $config) {

	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - modifier l'identifiant de l'arrondissement
	$element['com'] = 'FR-' . $element['com'];
	// - choisir le "bon" parent et modifier son identifiant
	if ($element['comparent']) {
		$element['arr'] = 'FR-' . $element['comparent'];
	} else {
		$element['arr'] = 'FR-' . $element['arr'];
	}
	// - affiner l'identifiant du type de commune
	switch ($element['typecom']) {
		case 'COMD':
			$element['typecom'] = 'delegated_commune';
			break;
		case 'COMA':
			$element['typecom'] = 'associated_commune';
			break;
		case 'ARM':
			$element['typecom'] = 'municipal_arrondissement';
			break;
		default:
			$element['typecom'] = 'commune';
	}

	return $element;
}
